"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DesktopNotification = exports.terminateNotifications = exports.initializeNotifications = void 0;
const uuid_1 = require("uuid");
const nativeModule = process.platform === 'win32'
    ? require('../../build/Release/desktop-notifications.node')
    : null;
function initializeNotifications(toastActivatorClsid) {
    if (!nativeModule) {
        // this code is a no-op when the module is missing
        return;
    }
    nativeModule.initializeNotifications(toastActivatorClsid);
}
exports.initializeNotifications = initializeNotifications;
function terminateNotifications() {
    if (!nativeModule) {
        // this code is a no-op when the module is missing
        return;
    }
    nativeModule.terminateNotifications();
}
exports.terminateNotifications = terminateNotifications;
function showNotification(id, title, body, callback) {
    if (!nativeModule) {
        // this code is a no-op when the module is missing
        return null;
    }
    return nativeModule.showNotification(id, title, body, callback);
}
function closeNotification(id) {
    if (!nativeModule) {
        // this code is a no-op when the module is missing
        return;
    }
    return nativeModule.closeNotification(id);
}
class DesktopNotification {
    constructor(title, body) {
        this.title = title;
        this.body = body;
        this.id = uuid_1.v4();
    }
    show() {
        showNotification(this.id, this.title, this.body, (event) => {
            var _a;
            if (event === 'click') {
                (_a = this.onclick) === null || _a === void 0 ? void 0 : _a.call(this);
            }
        });
    }
    close() {
        closeNotification(this.id);
    }
}
exports.DesktopNotification = DesktopNotification;
//# sourceMappingURL=desktop-notifications.js.map